@@ -14,7 +14,7 @@ from sklearn.isotonic import IsotonicRegression
 from skgstat import estimators, models, binning
 from skgstat import plotting
 from skgstat.util import shannon_entropy
-from .MetricSpace import MetricSpace, MetricSpacePair, ProbabalisticMetricSpace
+from .MetricSpace import MetricSpace, ProbabalisticMetricSpace
 from skgstat.interfaces.gstools import skgstat_to_gstools, skgstat_to_krige
 
 
@@ -116,12 +116,17 @@ class Variogram(object):
                 * `'fd'` applies Freedman-Diaconis estimator to find `n_lags`
                 * `'sturges'` applies Sturge's rule to find `n_lags`.
                 * `'scott'` applies Scott's rule to find `n_lags`
-                * `'doane'` applies Doane's extension to Sturge's rule to find `n_lags`
-                * `'sqrt'` uses the square-root of :func:`distance <skgstat.Variogram.distance>` as `n_lags`.
+                * `'doane'` applies Doane's extension to Sturge's rule
+                  to find `n_lags`
+                * `'sqrt'` uses the square-root of
+                  :func:`distance <skgstat.Variogram.distance>`
+                  as `n_lags`.
                 * `'kmeans'` uses KMeans clustering to well supported bins
-                * `'ward'` uses hierachical clustering to find minimum-variance clusters.
+                * `'ward'` uses hierachical clustering to find
+                  minimum-variance clusters.
 
-            More details are given in the documentation for :func:`set_bin_func <skgstat.Variogram.set_bin_func>`.
+            More details are given in the documentation for
+            :func:`set_bin_func <skgstat.Variogram.set_bin_func>`.
 
         normalize : bool
             Defaults to False. If True, the independent and dependent
@@ -140,15 +145,15 @@ class Variogram(object):
                 * 'trf': Trust Region Reflective function for non-linear
                   constrained problems. The class will set the boundaries
                   itself. This is the default function.
-                * 'ml': Maximum-Likelihood estimation. With the current implementation
-                  only the Nelder-Mead solver for unconstrained problems is
-                  implemented. This will estimate the variogram parameters from
-                  a Gaussian parameter space by minimizing the negative
-                  log-likelihood.
+                * 'ml': Maximum-Likelihood estimation. With the current
+                  implementation only the Nelder-Mead solver for
+                  unconstrained problems is implemented. This will estimate
+                  the variogram parameters from a Gaussian parameter space
+                  by minimizing the negative log-likelihood.
                 * 'manual': Manual fitting. You can set the range, sill and
-                  nugget either directly to the :func:`fit <skgstat.Variogram.fit>`
-                  function, or as `fit_` prefixed keyword arguments on
-                  Variogram instantiation.
+                  nugget either directly to the
+                  :func:`fit <skgstat.Variogram.fit>` function, or as
+                  `fit_` prefixed keyword arguments on Variogram instantiation.
 
         fit_sigma : numpy.ndarray, str
             Defaults to None. The sigma is used as measure of uncertainty
@@ -222,6 +227,14 @@ class Variogram(object):
             If :func:`bin_func <skgstat.Variogram.set_bin_func>` is `'ward'`
             this keyword argument can switch from default mean aggregation to
             median aggregation for calculating the cluster centroids.
+        obs_sigma : int, float
+            .. versionadded:: 0.6.0
+
+            If set, the Variogram will use this sigma as the standard deviation
+            of the observations passed as values. Using a MonteCarlo simulation
+            the uncertainties are propagated into the experimental variogram.
+            If present, the plot will indicate the confidence interval as
+            error bars around the experimental variogram.
 
         """
         # Before we do anything else, make kwargs available
@@ -239,13 +252,26 @@ class Variogram(object):
                     np.zeros(len(coordinates))
                 ))
                 self._1d = True
+
             # handle maxlag for MetricSpace
-            _maxlag = maxlag if maxlag and not isinstance(maxlag, str) and maxlag >= 1 else None
-            if samples == None:
-                coordinates = MetricSpace(coordinates.copy(), dist_func, _maxlag)
+            if maxlag and not isinstance(maxlag, str) and maxlag >= 1:
+                _maxlag = maxlag
             else:
-                coordinates = ProbabalisticMetricSpace(coordinates.copy(), dist_func, _maxlag, samples=samples,
-                                                       rnd=self._kwargs.get("binning_random_state", None))
+                _maxlag = None
+
+            if samples is None:
+                coordinates = MetricSpace(
+                    coordinates.copy(),
+                    dist_func,
+                    _maxlag
+                )
+            else:
+                coordinates = ProbabalisticMetricSpace(
+                    coordinates.copy(),
+                    dist_func, _maxlag,
+                    samples=samples,
+                    rnd=self._kwargs.get("binning_random_state", None)
+                )
         elif dist_func != coordinates.dist_metric:
             raise AttributeError((
                 "Distance metric of variogram differs "
@@ -260,7 +286,7 @@ class Variogram(object):
 
         # set verbosity
         self.verbose = verbose
-    
+
         # set values
         self._values = None
         # calc_diff = False here, because it will be calculated by fit() later
@@ -317,6 +343,12 @@ class Variogram(object):
         # Note that fit() calls preprocessing
         self.fit(force=True)
 
+        # finally check if any of the uncertainty propagation kwargs are set
+        self._experimental_conf_interval = None
+        self._model_conf_interval = None
+        if 'obs_sigma' in self._kwargs:
+            self._propagate_obs_sigma()
+
     @property
     def coordinates(self):
         """Coordinates property
@@ -332,6 +364,31 @@ class Variogram(object):
         """
         return self._X.coords
 
+    @property
+    def metric_space(self):
+        r"""
+        .. versionadded:: 0.5.6
+        
+        :class:`MetricSpace <skgstat.MetricSpace>` representation of the
+        input coordinates. A :class:`MetricSpace <skgstat.MetricSpace>`
+        can be used to pass pre-calculated coordinates to other
+        :class:`Variogram <skgstat.Variogram>` instances.
+
+        Returns
+        -------
+        metric_space : skgstat.MetricSpace
+
+        See Also
+        --------
+        Variogram.coordinates : coordinate representation
+
+        """
+        return self._X
+
+    @metric_space.setter
+    def metric_space(self, value):
+        raise AttributeError('MetricSpace is read-only.')
+
     @property
     def dim(self):
         """
@@ -527,7 +584,8 @@ class Variogram(object):
         .. math::
             n = log_2 n + 1
 
-        **`'scott'`**: estimates the lag class widths (h) by Scott's rule [102]_:
+        **`'scott'`**: estimates the lag class widths (h) by
+        Scott's rule [102]_:
 
         .. math::
             h = \sigma \frac{24 * \sqrt{\pi}}{n}^{\frac{1}{3}}
@@ -583,30 +641,32 @@ class Variogram(object):
 
         References
         ----------
-        .. [101] Scott, D.W. (2009), Sturges' rule. WIREs Comp Stat, 1: 303-306.
-            https://doi.org/10.1002/wics.35
-        .. [102] Scott, D.W. (2010), Scott's rule. WIREs Comp Stat, 2: 497-502.
-            https://doi.org/10.1002/wics.103
-        .. [103] Freedman, David, and Persi Diaconis  (1981), "On the histogram as
-            a density estimator: L 2 theory." Zeitschrift für Wahrscheinlichkeitstheorie
-            und verwandte Gebiete 57.4: 453-476.
+        .. [101] Scott, D.W. (2009), Sturges' rule. WIREs Comp Stat, 1:
+            303-306. https://doi.org/10.1002/wics.35
+        .. [102] Scott, D.W. (2010), Scott's rule. WIREs Comp Stat, 2:
+            497-502. https://doi.org/10.1002/wics.103
+        .. [103] Freedman, David, and Persi Diaconis  (1981), "On the
+            histogram as a density estimator: L 2 theory." Zeitschrift
+            für Wahrscheinlichkeitstheorie und verwandte Gebiete 57.4:
+            453-476.
         .. [104] Doane, D. P. (1976). Aesthetic frequency classifications.
             The American Statistician, 30(4), 181-183.
 
         """
         # handle strings
         if isinstance(bin_func, str):
+            fname = bin_func.lower()
             # switch the input
-            if bin_func.lower() == 'even':
+            if fname == 'even':
                 self._bin_func = binning.even_width_lags
 
-            elif bin_func.lower() == 'uniform':
+            elif fname == 'uniform':
                 self._bin_func = binning.uniform_count_lags
 
             # remove the n_lags if they will be adjusted on call
             else:
                 # reset lags for adjusting algorithms
-                if bin_func.lower() not in ('kmeans', 'ward', 'stable_entropy'):
+                if fname not in ('kmeans', 'ward', 'stable_entropy'):
                     self._n_lags = None
 
                 # use the wrapper for all but even and uniform
@@ -641,10 +701,19 @@ class Variogram(object):
             return binning.ward(distances, n, maxlag)
 
         elif self._bin_func_name.lower() == 'stable_entropy':
-            return binning.stable_entropy_lags(distances, n, maxlag, **self._kwargs)
+            return binning.stable_entropy_lags(
+                distances,
+                n,
+                maxlag,
+                **self._kwargs
+            )
 
         else:
-            return binning.auto_derived_lags(distances, self._bin_func_name, maxlag)
+            return binning.auto_derived_lags(
+                distances,
+                self._bin_func_name,
+                maxlag
+            )
 
     @property
     def normalized(self):
@@ -919,7 +988,7 @@ class Variogram(object):
         # handle sparse matrix
         if isinstance(self.distance_matrix, sparse.spmatrix):
             return self.triangular_distance_matrix.data
-        
+
         # Turn it back to triangular form not to have duplicates
         return squareform(self.distance_matrix)
 
@@ -936,7 +1005,10 @@ class Variogram(object):
         c = m.tocsc()
         c.data = c.indices
         rows = c.tocsr()
-        filt = sparse.csr.csr_matrix((m.indices < rows.data, m.indices, m.indptr), m.shape)
+        filt = sparse.csr.csr_matrix(
+            (m.indices < rows.data, m.indices, m.indptr),
+            m.shape
+        )
         return m.multiply(filt)
 
     @property
@@ -955,7 +1027,7 @@ class Variogram(object):
         # remove bins
         self._bins = None
         self._groups = None
-        
+
         # set new maxlag
         if value is None:
             self._maxlag = None
@@ -978,7 +1050,8 @@ class Variogram(object):
         cost function, which divides the residuals by their uncertainty.
 
         When setting fit_sigma, the array of uncertainties itself can be
-        given, or one of the strings: ['linear', 'exp', 'sqrt', 'sq', 'entropy'].
+        given, or one of the strings:
+        ['linear', 'exp', 'sqrt', 'sq', 'entropy'].
         The parameters described below refer to the setter of this property.
 
         .. versionchanged:: 0.3.11
@@ -1034,7 +1107,7 @@ class Variogram(object):
             if len(self._fit_sigma) == len(self._bins):
                 return self._fit_sigma
             else:
-                raise AttributeError('fit_sigma and bins need the same length.')
+                raise AttributeError('len(fit_sigma) != len(bins)')
 
         # linear function of distance
         elif self._fit_sigma == 'linear':
@@ -1208,15 +1281,15 @@ class Variogram(object):
                 scipy.optimize.leastsq function.
               * trf: Trust Region Reflective algorithm implemented in
                 scipy.optimize.least_squares(method='trf')
-              * 'ml': Maximum-Likelihood estimation. With the current implementation
-                only the Nelder-Mead solver for unconstrained problems is
-                implemented. This will estimate the variogram parameters from
-                a Gaussian parameter space by minimizing the negative
-                log-likelihood.
+              * 'ml': Maximum-Likelihood estimation. With the current
+                implementation only the Nelder-Mead solver for unconstrained
+                problems is implemented. This will estimate the variogram
+                parameters from a Gaussian parameter space by minimizing
+                the negative log-likelihood.
               * 'manual': Manual fitting. You can set the range, sill and
-                nugget either directly to the :func:`fit <skgstat.Variogram.fit>`
-                function, or as `fit_` prefixed keyword arguments on
-                Variogram instantiation.
+                nugget either directly to the
+                :func:`fit <skgstat.Variogram.fit>` function, or as
+                `fit_` prefixed keyword arguments on Variogram instantiation.
 
 
         sigma : string, array
@@ -1436,14 +1509,13 @@ class Variogram(object):
             if kw["nugget"] != 0:
                 cof.append(kw["nugget"])
 
-        harmonize = False
         if not callable(model):
             if model == "harmonize":
                 raise ValueError("Please supply the actual harmonized model directly")
             else:
                 model = model.lower()
                 model = getattr(models, model)
-            
+
         if model.__name__ == "harmonize":
             code = """fitted_model = lambda x: model(x)"""
         else:
@@ -1491,7 +1563,7 @@ class Variogram(object):
             self._diff = np.abs(Vrow.data - Vcol.data)
         else:
             # Append a column of zeros to make pdist happy
-            # euclidean: sqrt((a-b)**2 + (0-0)**2) == sqrt((a-b)**2) == abs(a-b)
+            # euclidean: sqrt((a-b)**2 + (0-0)**2) == sqrt((a-b)**2)
             self._diff = pdist(
                 np.column_stack((v, np.zeros(len(v)))),
                 metric="euclidean"
@@ -1715,7 +1787,6 @@ class Variogram(object):
         self.preprocessing(force=force)
 
         # calculate the experimental variogram
-        _exp = self.experimental
         _bin = self.bins
 
         x = np.linspace(0, np.float64(np.nanmax(_bin)), n)
@@ -1726,6 +1797,22 @@ class Variogram(object):
 
         return x, self._model(x, *self.cof)
 
+    def _propagate_obs_sigma(self):
+        """
+        Observation uncertainties are propagated into the experimental
+        variogram.
+        """
+        # due to circular import, import here
+        from skgstat.util.uncertainty import propagate
+
+        # TODO: load and propagate percentile settings
+        self._experimental_conf_interval = propagate(
+            self,
+            'values',
+            self._kwargs['obs_sigma'],
+            use_bounds=True
+        )
+
     @property
     def residuals(self):
         """Model residuals
@@ -1858,7 +1945,6 @@ class Variogram(object):
 
         return mae
 
-
     @property
     def nrmse(self):
         r"""NRMSE
@@ -1957,8 +2043,16 @@ class Variogram(object):
         mx = np.nanmean(experimental)
 
         # calculate the single nash-sutcliffe terms
-        term1 = np.nansum(np.fromiter(map(lambda x, y: (x - y)**2, experimental, model), float))
-        term2 = np.nansum(np.fromiter(map(lambda x: (x - mx)**2, experimental), float))
+        term1 = np.nansum(np.fromiter(
+            map(lambda x, y: (x - y)**2, experimental, model),
+            float
+            )
+        )
+        term2 = np.nansum(np.fromiter(
+            map(lambda x: (x - mx)**2, experimental),
+            float
+            )
+        )
 
         return 1 - (term1 / term2)
 
@@ -2013,7 +2107,7 @@ class Variogram(object):
             The number of points to be included into the cross-validation.
             If None (default), all points will be used.
         metric : str
-            Metric used for cross-validation. 
+            Metric used for cross-validation.
             Can be root mean square error (rmse), mean squared error (mse)
             or mean absolute error (mae).
         seed : int
@@ -2033,7 +2127,12 @@ class Variogram(object):
         from skgstat.util import cross_validation
 
         if method == 'jacknife':
-            return cross_validation.jacknife(self, n=n, metric=metric, seed=seed)
+            return cross_validation.jacknife(
+                self,
+                n=n,
+                metric=metric,
+                seed=seed
+            )
         else:
             raise AttributeError(f"A method '{method}' is not implemented.")
 
@@ -2139,9 +2238,16 @@ class Variogram(object):
     @property
     def parameters(self):
         """
-        Extract just the variogram parameters range, sill and nugget from the self.describe return
+        Extract just the variogram parameters range, sill
+        and nugget from the
+        :func:`describe <skgstat.Variogram.describe>` output.
+
+        Returns
+        -------
+        params : list
+            [range, sill, nugget] for most models and
+            [range, sill, shape, nugget] for matern and stable model.
 
-        :return:
         """
         d = self.describe()
         if 'error' in d:
@@ -2322,9 +2428,21 @@ class Variogram(object):
         used_backend = plotting.backend()
 
         if used_backend == 'matplotlib':
-            return plotting.matplotlib_variogram_plot(self, axes=axes, grid=grid, show=show, hist=hist)
+            return plotting.matplotlib_variogram_plot(
+                self,
+                axes=axes,
+                grid=grid,
+                show=show,
+                hist=hist
+            )
         elif used_backend == 'plotly':
-            return plotting.plotly_variogram_plot(self, fig=axes, grid=grid, show=show, hist=hist)
+            return plotting.plotly_variogram_plot(
+                self,
+                fig=axes,
+                grid=grid,
+                show=show,
+                hist=hist
+            )
 
         # if we reach this line, somethings wrong with plotting backend
         raise ValueError('The plotting backend has an undefined state.')
@@ -2360,9 +2478,19 @@ class Variogram(object):
         used_backend = plotting.backend()
 
         if used_backend == 'matplotlib':
-            return plotting.matplotlib_variogram_scattergram(self, ax=ax, show=show, **kwargs)
+            return plotting.matplotlib_variogram_scattergram(
+                self,
+                ax=ax,
+                show=show,
+                **kwargs
+            )
         elif used_backend == 'plotly':
-            return plotting.plotly_variogram_scattergram(self, fig=ax, show=show, **kwargs)
+            return plotting.plotly_variogram_scattergram(
+                self,
+                fig=ax,
+                show=show,
+                **kwargs
+            )
 
         # if we reach this line, somethings wrong with plotting backend
         raise ValueError('The plotting backend has an undefined state.')
@@ -2417,9 +2545,19 @@ class Variogram(object):
         used_backend = plotting.backend()
 
         if used_backend == 'matplotlib':
-            return plotting.matplotlib_location_trend(self, axes=axes, show=show, **kwargs)
+            return plotting.matplotlib_location_trend(
+                self,
+                axes=axes,
+                show=show,
+                **kwargs
+            )
         elif used_backend == 'plotly':
-            return plotting.plotly_location_trend(self, fig=axes, show=show, **kwargs)
+            return plotting.plotly_location_trend(
+                self,
+                fig=axes,
+                show=show,
+                **kwargs
+            )
 
         # if we reach this line, somethings wrong with plotting backend
         raise ValueError('The plotting backend has an undefined state.')
@@ -2454,9 +2592,19 @@ class Variogram(object):
         used_backend = plotting.backend()
 
         if used_backend == 'matplotlib':
-            return plotting.matplotlib_dd_plot(self, ax=ax, plot_bins=plot_bins, show=show)
+            return plotting.matplotlib_dd_plot(
+                self,
+                ax=ax,
+                plot_bins=plot_bins,
+                show=show
+            )
         elif used_backend == 'plotly':
-            return plotting.plotly_dd_plot(self, fig=ax, plot_bins=plot_bins, show=show)
+            return plotting.plotly_dd_plot(
+                self,
+                fig=ax,
+                plot_bins=plot_bins,
+                show=show
+            )
 
         # if we reach this line, somethings wrong with plotting backend
         raise ValueError('The plotting backend has an undefined state.')
@@ -2470,7 +2618,7 @@ class Variogram(object):
         try:
             _name = self._model.__name__
             _b = int(len(self.bins))
-        except:
+        except Exception:
             return "< abstract Variogram >"
         return "< %s Semivariogram fitted to %d bins >" % (_name, _b)
 
@@ -2494,8 +2642,8 @@ class Variogram(object):
         _nugget = np.NaN if 'error' in par else par['nugget']
 
         s = "{0} Variogram\n".format(par['model'])
-        s+= "-" * (len(s) - 1) + "\n"
-        s+="""Estimator:         %s
+        s += "-" * (len(s) - 1) + "\n"
+        s += """Estimator:         %s
         \rEffective Range:   %.2f
         \rSill:              %.2f
         \rNugget:            %.2f
